@@ -1,15 +1,18 @@
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
 import functools
 import operator
 from collections import defaultdict
+import warnings
 
 import numpy as np
-import pandas as pd
 
-from . import ops, utils
-from .common import _maybe_promote
-from .pycompat import iteritems, OrderedDict
+from . import utils
+from .indexing import get_indexer_nd
+from .pycompat import iteritems, OrderedDict, suppress
 from .utils import is_full_slice, is_dict_like
-from .variable import Variable, Coordinate
+from .variable import IndexVariable
 
 
 def _get_joiner(join):
@@ -21,18 +24,26 @@ def _get_joiner(join):
         return operator.itemgetter(0)
     elif join == 'right':
         return operator.itemgetter(-1)
+    elif join == 'exact':
+        # We cannot return a function to "align" in this case, because it needs
+        # access to the dimension name to give a good error message.
+        return None
     else:
         raise ValueError('invalid value for join: %s' % join)
 
 
+_DEFAULT_EXCLUDE = frozenset()
+
+
 def align(*objects, **kwargs):
-    """align(*objects, join='inner', copy=True)
+    """align(*objects, join='inner', copy=True, indexes=None,
+             exclude=frozenset())
 
     Given any number of Dataset and/or DataArray objects, returns new
-    objects with aligned indexes.
+    objects with aligned indexes and dimension sizes.
 
     Array from the aligned objects are suitable as input to mathematical
-    operators, because along each dimension they have the same indexes.
+    operators, because along each dimension they have the same index and size.
 
     Missing values (if ``join != 'inner'``) are filled with NaN.
 
@@ -40,13 +51,15 @@ def align(*objects, **kwargs):
     ----------
     *objects : Dataset or DataArray
         Objects to align.
-    join : {'outer', 'inner', 'left', 'right'}, optional
+    join : {'outer', 'inner', 'left', 'right', 'exact'}, optional
         Method for joining the indexes of the passed objects along each
         dimension:
         - 'outer': use the union of object indexes
         - 'inner': use the intersection of object indexes
         - 'left': use indexes from the first object with each dimension
         - 'right': use indexes from the last object with each dimension
+        - 'exact': instead of aligning, raise `ValueError` when indexes to be
+          aligned are not equal
     copy : bool, optional
         If ``copy=True``, data in the return values is always copied. If
         ``copy=False`` and reindexing is unnecessary, or can be performed with
@@ -62,64 +75,205 @@ def align(*objects, **kwargs):
     -------
     aligned : same as *objects
         Tuple of objects with aligned coordinates.
+
+    Raises
+    ------
+    ValueError
+        If any dimensions without labels on the arguments have different sizes,
+        or a different size than the size of the aligned dimension labels.
     """
     join = kwargs.pop('join', 'inner')
     copy = kwargs.pop('copy', True)
     indexes = kwargs.pop('indexes', None)
-    exclude = kwargs.pop('exclude', None)
+    exclude = kwargs.pop('exclude', _DEFAULT_EXCLUDE)
     if indexes is None:
         indexes = {}
-    if exclude is None:
-        exclude = set()
     if kwargs:
         raise TypeError('align() got unexpected keyword arguments: %s'
                         % list(kwargs))
 
-    all_indexes = defaultdict(list)
-    for obj in objects:
-        for dim, index in iteritems(obj.indexes):
-            if dim not in exclude:
-                all_indexes[dim].append(index)
+    if not indexes and len(objects) == 1:
+        # fast path for the trivial case
+        obj, = objects
+        return (obj.copy(deep=copy),)
 
-    # We don't join over dimensions with all equal indexes for two reasons:
+    all_indexes = defaultdict(list)
+    unlabeled_dim_sizes = defaultdict(set)
+    for obj in objects:
+        for dim in obj.dims:
+            if dim not in exclude:
+                try:
+                    index = obj.indexes[dim]
+                except KeyError:
+                    unlabeled_dim_sizes[dim].add(obj.sizes[dim])
+                else:
+                    all_indexes[dim].append(index)
+
+    # We don't reindex over dimensions with all equal indexes for two reasons:
     # - It's faster for the usual case (already aligned objects).
     # - It ensures it's possible to do operations that don't require alignment
     #   on indexes with duplicate values (which cannot be reindexed with
     #   pandas). This is useful, e.g., for overwriting such duplicate indexes.
     joiner = _get_joiner(join)
     joined_indexes = {}
-    for dim, dim_indexes in iteritems(all_indexes):
+    for dim, matching_indexes in iteritems(all_indexes):
         if dim in indexes:
             index = utils.safe_cast_to_index(indexes[dim])
-            if any(not index.equals(other) for other in dim_indexes):
+            if (any(not index.equals(other) for other in matching_indexes) or
+                    dim in unlabeled_dim_sizes):
                 joined_indexes[dim] = index
         else:
-            if any(not dim_indexes[0].equals(other)
-                   for other in dim_indexes[1:]):
-                joined_indexes[dim] = joiner(dim_indexes)
+            if (any(not matching_indexes[0].equals(other)
+                    for other in matching_indexes[1:]) or
+                    dim in unlabeled_dim_sizes):
+                if join == 'exact':
+                    raise ValueError(
+                        'indexes along dimension {!r} are not equal'
+                        .format(dim))
+                index = joiner(matching_indexes)
+                joined_indexes[dim] = index
+            else:
+                index = matching_indexes[0]
+
+        if dim in unlabeled_dim_sizes:
+            unlabeled_sizes = unlabeled_dim_sizes[dim]
+            labeled_size = index.size
+            if len(unlabeled_sizes | {labeled_size}) > 1:
+                raise ValueError(
+                    'arguments without labels along dimension %r cannot be '
+                    'aligned because they have different dimension size(s) %r '
+                    'than the size of the aligned dimension labels: %r'
+                    % (dim, unlabeled_sizes, labeled_size))
+
+    for dim in unlabeled_dim_sizes:
+        if dim not in all_indexes:
+            sizes = unlabeled_dim_sizes[dim]
+            if len(sizes) > 1:
+                raise ValueError(
+                    'arguments without labels along dimension %r cannot be '
+                    'aligned because they have different dimension sizes: %r'
+                    % (dim, sizes))
 
     result = []
     for obj in objects:
-        valid_indexers = dict((k, v) for k, v in joined_indexes.items()
-                              if k in obj.dims)
-        result.append(obj.reindex(copy=copy, **valid_indexers))
+        valid_indexers = {k: v for k, v in joined_indexes.items()
+                          if k in obj.dims}
+        if not valid_indexers:
+            # fast path for no reindexing necessary
+            new_obj = obj.copy(deep=copy)
+        else:
+            new_obj = obj.reindex(copy=copy, **valid_indexers)
+        new_obj.encoding = obj.encoding
+        result.append(new_obj)
 
     return tuple(result)
 
 
-def reindex_variables(variables, indexes, indexers, method=None,
+def deep_align(objects, join='inner', copy=True, indexes=None,
+               exclude=frozenset(), raise_on_invalid=True):
+    """Align objects for merging, recursing into dictionary values.
+
+    This function is not public API.
+    """
+    if indexes is None:
+        indexes = {}
+
+    def is_alignable(obj):
+        return hasattr(obj, 'indexes') and hasattr(obj, 'reindex')
+
+    positions = []
+    keys = []
+    out = []
+    targets = []
+    no_key = object()
+    not_replaced = object()
+    for n, variables in enumerate(objects):
+        if is_alignable(variables):
+            positions.append(n)
+            keys.append(no_key)
+            targets.append(variables)
+            out.append(not_replaced)
+        elif is_dict_like(variables):
+            for k, v in variables.items():
+                if is_alignable(v) and k not in indexes:
+                    # Skip variables in indexes for alignment, because these
+                    # should to be overwritten instead:
+                    # https://github.com/pydata/xarray/issues/725
+                    positions.append(n)
+                    keys.append(k)
+                    targets.append(v)
+            out.append(OrderedDict(variables))
+        elif raise_on_invalid:
+            raise ValueError('object to align is neither an xarray.Dataset, '
+                             'an xarray.DataArray nor a dictionary: %r'
+                             % variables)
+        else:
+            out.append(variables)
+
+    aligned = align(*targets, join=join, copy=copy, indexes=indexes,
+                    exclude=exclude)
+
+    for position, key, aligned_obj in zip(positions, keys, aligned):
+        if key is no_key:
+            out[position] = aligned_obj
+        else:
+            out[position][key] = aligned_obj
+
+    # something went wrong: we should have replaced all sentinel values
+    assert all(arg is not not_replaced for arg in out)
+
+    return out
+
+
+def reindex_like_indexers(target, other):
+    """Extract indexers to align target with other.
+
+    Not public API.
+
+    Parameters
+    ----------
+    target : Dataset or DataArray
+        Object to be aligned.
+    other : Dataset or DataArray
+        Object to be aligned with.
+
+    Returns
+    -------
+    Dict[Any, pandas.Index] providing indexes for reindex keyword arguments.
+
+    Raises
+    ------
+    ValueError
+        If any dimensions without labels have different sizes.
+    """
+    indexers = {k: v for k, v in other.indexes.items() if k in target.dims}
+
+    for dim in other.dims:
+        if dim not in indexers and dim in target.dims:
+            other_size = other.sizes[dim]
+            target_size = target.sizes[dim]
+            if other_size != target_size:
+                raise ValueError('different size for unlabeled '
+                                 'dimension on argument %r: %r vs %r'
+                                 % (dim, other_size, target_size))
+    return indexers
+
+
+def reindex_variables(variables, sizes, indexes, indexers, method=None,
                       tolerance=None, copy=True):
     """Conform a dictionary of aligned variables onto a new set of variables,
     filling in missing values with NaN.
 
-    WARNING: This method is not public API. Don't use it directly.
+    Not public API.
 
     Parameters
     ----------
     variables : dict-like
         Dictionary of xarray.Variable objects.
+    sizes : dict-like
+        Dictionary from dimension names to integer sizes.
     indexes : dict-like
-        Dictionary of xarray.Coordinate objects associated with variables.
+        Dictionary of xarray.IndexVariable objects associated with variables.
     indexers : dict
         Dictionary with keys given by dimension names and values given by
         arrays of coordinates tick labels. Any mis-matched coordinate values
@@ -147,102 +301,88 @@ def reindex_variables(variables, indexes, indexers, method=None,
     reindexed : OrderedDict
         Another dict, with the items in variables but replaced indexes.
     """
-    # build up indexers for assignment along each index
-    to_indexers = {}
-    to_shape = {}
-    from_indexers = {}
+    from .dataarray import DataArray
 
-    # for compat with older versions of pandas that don't support tolerance
-    get_indexer_kwargs = {}
-    if tolerance is not None:
-        if pd.__version__ < '0.17':
-            raise NotImplementedError(
-                'the tolerance argument requires pandas v0.17 or newer')
-        get_indexer_kwargs['tolerance'] = tolerance
+    # build up indexers for assignment along each dimension
+    int_indexers = {}
+    targets = {}
+    masked_dims = set()
+    unchanged_dims = set()
+
+    # size of reindexed dimensions
+    new_sizes = {}
 
     for name, index in iteritems(indexes):
-        to_shape[name] = index.size
         if name in indexers:
-            target = utils.safe_cast_to_index(indexers[name])
             if not index.is_unique:
                 raise ValueError(
                     'cannot reindex or align along dimension %r because the '
                     'index has duplicate values' % name)
-            indexer = index.get_indexer(target, method=method,
-                                        **get_indexer_kwargs)
 
-            to_shape[name] = len(target)
-            # Note pandas uses negative values from get_indexer to signify
-            # values that are missing in the index
-            # The non-negative values thus indicate the non-missing values
-            to_indexers[name] = indexer >= 0
-            if to_indexers[name].all():
-                # If an indexer includes no negative values, then the
-                # assignment can be to a full-slice (which is much faster,
-                # and means we won't need to fill in any missing values)
-                to_indexers[name] = slice(None)
+            target = utils.safe_cast_to_index(indexers[name])
+            new_sizes[name] = len(target)
 
-            from_indexers[name] = indexer[to_indexers[name]]
-            if np.array_equal(from_indexers[name], np.arange(index.size)):
-                # If the indexer is equal to the original index, use a full
-                # slice object to speed up selection and so we can avoid
-                # unnecessary copies
-                from_indexers[name] = slice(None)
+            int_indexer = get_indexer_nd(index, target, method, tolerance)
 
-    def any_not_full_slices(indexers):
-        return any(not is_full_slice(idx) for idx in indexers)
+            # We uses negative values from get_indexer_nd to signify
+            # values that are missing in the index.
+            if (int_indexer < 0).any():
+                masked_dims.add(name)
+            elif np.array_equal(int_indexer, np.arange(len(index))):
+                unchanged_dims.add(name)
 
-    def var_indexers(var, indexers):
-        return tuple(indexers.get(d, slice(None)) for d in var.dims)
+            int_indexers[name] = int_indexer
+            targets[name] = target
+
+    for dim in sizes:
+        if dim not in indexes and dim in indexers:
+            existing_size = sizes[dim]
+            new_size = indexers[dim].size
+            if existing_size != new_size:
+                raise ValueError(
+                    'cannot reindex or align along dimension %r without an '
+                    'index because its size %r is different from the size of '
+                    'the new index %r' % (dim, existing_size, new_size))
 
     # create variables for the new dataset
     reindexed = OrderedDict()
-    for name, var in iteritems(variables):
-        if name in indexers:
-            # no need to copy, because index data is immutable
-            new_var = Coordinate(var.dims, indexers[name], var.attrs,
-                                 var.encoding)
+
+    for dim, indexer in indexers.items():
+        if isinstance(indexer, DataArray) and indexer.dims != (dim,):
+            warnings.warn(
+                "Indexer has dimensions {0:s} that are different "
+                "from that to be indexed along {1:s}. "
+                "This will behave differently in the future.".format(
+                    str(indexer.dims), dim),
+                FutureWarning, stacklevel=3)
+
+        if dim in variables:
+            var = variables[dim]
+            args = (var.attrs, var.encoding)
         else:
-            assign_to = var_indexers(var, to_indexers)
-            assign_from = var_indexers(var, from_indexers)
+            args = ()
+        reindexed[dim] = IndexVariable((dim,), indexers[dim], *args)
 
-            if any_not_full_slices(assign_to):
-                # there are missing values to in-fill
-                data = var[assign_from].data
-                dtype, fill_value = _maybe_promote(var.dtype)
+    for name, var in iteritems(variables):
+        if name not in indexers:
+            key = tuple(slice(None)
+                        if d in unchanged_dims
+                        else int_indexers.get(d, slice(None))
+                        for d in var.dims)
+            needs_masking = any(d in masked_dims for d in var.dims)
 
-                if isinstance(data, np.ndarray):
-                    shape = tuple(to_shape[dim] for dim in var.dims)
-                    new_data = np.empty(shape, dtype=dtype)
-                    new_data[...] = fill_value
-                    # create a new Variable so we can use orthogonal indexing
-                    # use fastpath=True to avoid dtype inference
-                    new_var = Variable(var.dims, new_data, var.attrs,
-                                       fastpath=True)
-                    new_var[assign_to] = data
-
-                else:  # dask array
-                    data = data.astype(dtype, copy=False)
-                    for axis, indexer in enumerate(assign_to):
-                        if not is_full_slice(indexer):
-                            indices = np.cumsum(indexer)[~indexer]
-                            data = ops.insert(data, indices, fill_value,
-                                              axis=axis)
-                    new_var = Variable(var.dims, data, var.attrs,
-                                       fastpath=True)
-
-            elif any_not_full_slices(assign_from):
-                # type coercion is not necessary as there are no missing
-                # values
-                new_var = var[assign_from]
-
-            else:
-                # no reindexing is necessary
+            if needs_masking:
+                new_var = var._getitem_with_mask(key)
+            elif all(is_full_slice(k) for k in key):
+                # no reindexing necessary
                 # here we need to manually deal with copying data, since
                 # we neither created a new ndarray nor used fancy indexing
                 new_var = var.copy(deep=copy)
+            else:
+                new_var = var[key]
+
+            reindexed[name] = new_var
 
-        reindexed[name] = new_var
     return reindexed
 
 
@@ -269,11 +409,6 @@ def broadcast(*args, **kwargs):
         The same data as the input arrays, but with additional dimensions
         inserted so that all data arrays have the same dimensions and shape.
 
-    Raises
-    ------
-    ValueError
-        If indexes on the different objects are not aligned.
-
     Examples
     --------
 
@@ -340,23 +475,22 @@ def broadcast(*args, **kwargs):
     for arg in args:
         for dim in arg.dims:
             if dim not in common_coords and dim not in exclude:
-                common_coords[dim] = arg.coords[dim].variable
-                dims_map[dim] = common_coords[dim].size
+                dims_map[dim] = arg.sizes[dim]
+                if dim in arg.coords:
+                    common_coords[dim] = arg.coords[dim].variable
 
-    def _expand_dims(var):
+    def _set_dims(var):
         # Add excluded dims to a copy of dims_map
         var_dims_map = dims_map.copy()
         for dim in exclude:
-            try:
+            with suppress(ValueError):
+                # ignore dim not in var.dims
                 var_dims_map[dim] = var.shape[var.dims.index(dim)]
-            except ValueError:
-                # dim not in var.dims
-                pass
 
-        return var.expand_dims(var_dims_map)
+        return var.set_dims(var_dims_map)
 
     def _broadcast_array(array):
-        data = _expand_dims(array.variable)
+        data = _set_dims(array.variable)
         coords = OrderedDict(array.coords)
         coords.update(common_coords)
         return DataArray(data, coords, data.dims, name=array.name,
@@ -364,7 +498,7 @@ def broadcast(*args, **kwargs):
 
     def _broadcast_dataset(ds):
         data_vars = OrderedDict(
-            (k, _expand_dims(ds.variables[k]))
+            (k, _set_dims(ds.variables[k]))
             for k in ds.data_vars)
         coords = OrderedDict(ds.coords)
         coords.update(common_coords)
