@@ -3,6 +3,8 @@ from __future__ import division
 from __future__ import print_function
 import functools
 import operator
+import warnings
+from distutils.version import LooseVersion
 
 import numpy as np
 
@@ -74,19 +76,28 @@ class NetCDF4ArrayWrapper(BaseNetCDF4Array):
         return data
 
 
-def _nc4_values_and_dtype(var):
+def _encode_nc4_variable(var):
+    if var.dtype.kind == 'S':
+        var = conventions.maybe_encode_as_char_array(var)
+    return var
+
+
+def _get_datatype(var, nc_format='NETCDF4'):
+    if nc_format == 'NETCDF4':
+        datatype = _nc4_dtype(var)
+    else:
+        datatype = var.dtype
+    return datatype
+
+
+def _nc4_dtype(var):
     if var.dtype.kind == 'U':
         dtype = str
-    elif var.dtype.kind == 'S':
-        # use character arrays instead of unicode, because unicode support in
-        # netCDF4 is still rather buggy
-        var = conventions.maybe_encode_as_char_array(var)
-        dtype = var.dtype
-    elif var.dtype.kind in ['i', 'u', 'f', 'c']:
+    elif var.dtype.kind in ['i', 'u', 'f', 'c', 'S']:
         dtype = var.dtype
     else:
         raise ValueError('cannot infer dtype for netCDF4 variable')
-    return var, dtype
+    return dtype
 
 
 def _nc4_group(ds, group, mode):
@@ -149,7 +160,7 @@ def _extract_nc4_variable_encoding(variable, raise_on_invalid=False,
 
     safe_to_drop = set(['source', 'original_shape'])
     valid_encodings = set(['zlib', 'complevel', 'fletcher32', 'contiguous',
-                           'chunksizes', 'shuffle'])
+                           'chunksizes', 'shuffle', '_FillValue'])
     if lsd_okay:
         valid_encodings.add('least_significant_digit')
 
@@ -218,6 +229,7 @@ class NetCDF4DataStore(WritableCFDataStore, DataStorePickleMixin):
 
     This store supports NetCDF3, NetCDF4 and OpenDAP datasets.
     """
+
     def __init__(self, netcdf4_dataset, mode='r', writer=None, opener=None,
                  autoclose=False):
 
@@ -243,6 +255,17 @@ class NetCDF4DataStore(WritableCFDataStore, DataStorePickleMixin):
     def open(cls, filename, mode='r', format='NETCDF4', group=None,
              writer=None, clobber=True, diskless=False, persist=False,
              autoclose=False):
+        import netCDF4 as nc4
+        if (len(filename) == 88 and
+                LooseVersion(nc4.__version__) < "1.3.1"):
+            warnings.warn(
+                '\nA segmentation fault may occur when the\n'
+                'file path has exactly 88 characters as it does.\n'
+                'in this case. The issue is known to occur with\n'
+                'version 1.2.4 of netCDF4 and can be addressed by\n'
+                'upgrading netCDF4 to at least version 1.3.1.\n'
+                'More details can be found here:\n'
+                'https://github.com/pydata/xarray/issues/1745  \n')
         if format is None:
             format = 'NETCDF4'
         opener = functools.partial(_open_netcdf4_group, filename, mode=mode,
@@ -324,18 +347,17 @@ class NetCDF4DataStore(WritableCFDataStore, DataStorePickleMixin):
         with self.ensure_open(autoclose=False):
             super(NetCDF4DataStore, self).set_variables(*args, **kwargs)
 
-    def prepare_variable(self, name, variable, check_encoding=False,
-                         unlimited_dims=None):
+    def encode_variable(self, variable):
         variable = _force_native_endianness(variable)
-
         if self.format == 'NETCDF4':
-            variable, datatype = _nc4_values_and_dtype(variable)
+            variable = _encode_nc4_variable(variable)
         else:
             variable = encode_nc3_variable(variable)
-            datatype = variable.dtype
-
-        self.set_necessary_dimensions(variable, unlimited_dims=unlimited_dims)
+        return variable
 
+    def prepare_variable(self, name, variable, check_encoding=False,
+                         unlimited_dims=None):
+        datatype = _get_datatype(variable, self.format)
         attrs = variable.attrs.copy()
 
         fill_value = attrs.pop('_FillValue', None)
@@ -352,20 +374,24 @@ class NetCDF4DataStore(WritableCFDataStore, DataStorePickleMixin):
         encoding = _extract_nc4_variable_encoding(
             variable, raise_on_invalid=check_encoding,
             unlimited_dims=unlimited_dims)
-        nc4_var = self.ds.createVariable(
-            varname=name,
-            datatype=datatype,
-            dimensions=variable.dims,
-            zlib=encoding.get('zlib', False),
-            complevel=encoding.get('complevel', 4),
-            shuffle=encoding.get('shuffle', True),
-            fletcher32=encoding.get('fletcher32', False),
-            contiguous=encoding.get('contiguous', False),
-            chunksizes=encoding.get('chunksizes'),
-            endian='native',
-            least_significant_digit=encoding.get('least_significant_digit'),
-            fill_value=fill_value)
-        _disable_auto_decode_variable(nc4_var)
+        if name in self.ds.variables:
+            nc4_var = self.ds.variables[name]
+        else:
+            nc4_var = self.ds.createVariable(
+                varname=name,
+                datatype=datatype,
+                dimensions=variable.dims,
+                zlib=encoding.get('zlib', False),
+                complevel=encoding.get('complevel', 4),
+                shuffle=encoding.get('shuffle', True),
+                fletcher32=encoding.get('fletcher32', False),
+                contiguous=encoding.get('contiguous', False),
+                chunksizes=encoding.get('chunksizes'),
+                endian='native',
+                least_significant_digit=encoding.get(
+                    'least_significant_digit'),
+                fill_value=fill_value)
+            _disable_auto_decode_variable(nc4_var)
 
         for k, v in iteritems(attrs):
             # set attributes one-by-one since netCDF4<1.0.10 can't handle
