@@ -4,7 +4,7 @@ from __future__ import print_function
 import numpy as np
 
 from .. import Variable
-from ..core.utils import FrozenOrderedDict, Frozen
+from ..core.utils import FrozenOrderedDict, Frozen, is_dict_like
 from ..core import indexing
 from ..core.pycompat import integer_types
 
@@ -15,16 +15,13 @@ class PydapArrayWrapper(BackendArray):
     def __init__(self, array):
         self.array = array
 
+    @property
+    def shape(self):
+        return self.array.shape
+
     @property
     def dtype(self):
-        t = self.array.type
-        if t.size is None and t.typecode == 'S':
-            # return object dtype because that's the only way in numpy to
-            # represent variable length strings; it also prevents automatic
-            # string concatenation via conventions.decode_cf_variable
-            return np.dtype('O')
-        else:
-            return np.dtype(t.typecode + str(t.size))
+        return self.array.dtype
 
     def __getitem__(self, key):
         key = indexing.unwrap_explicit_indexer(
@@ -37,17 +34,23 @@ class PydapArrayWrapper(BackendArray):
         # pydap doesn't squeeze axes automatically like numpy
         axis = tuple(n for n, k in enumerate(key)
                      if isinstance(k, integer_types))
-        result = np.squeeze(result, axis)
+        if len(axis) > 0:
+            result = np.squeeze(result, axis)
         return result
 
 
-def _fix_global_attributes(attributes):
+def _fix_attributes(attributes):
     attributes = dict(attributes)
     for k in list(attributes):
         if k.lower() == 'global' or k.lower().endswith('_global'):
             # move global attributes to the top level, like the netcdf-C
             # DAP client
             attributes.update(attributes.pop(k))
+        elif is_dict_like(attributes[k]):
+            # Make Hierarchical attributes to a single level with a
+            # dot-separated key
+            attributes.update({'{}.{}'.format(k, k_child): v_child for
+                               k_child, v_child in attributes.pop(k).items()})
     return attributes
 
 
@@ -57,6 +60,7 @@ class PydapDataStore(AbstractDataStore):
     This store provides an alternative way to access OpenDAP datasets that may
     be useful if the netCDF4 library is not available.
     """
+
     def __init__(self, ds):
         """
         Parameters
@@ -73,14 +77,15 @@ class PydapDataStore(AbstractDataStore):
 
     def open_store_variable(self, var):
         data = indexing.LazilyIndexedArray(PydapArrayWrapper(var))
-        return Variable(var.dimensions, data, var.attributes)
+        return Variable(var.dimensions, data,
+                        _fix_attributes(var.attributes))
 
     def get_variables(self):
         return FrozenOrderedDict((k, self.open_store_variable(self.ds[k]))
                                  for k in self.ds.keys())
 
     def get_attrs(self):
-        return Frozen(_fix_global_attributes(self.ds.attributes))
+        return Frozen(_fix_attributes(self.ds.attributes))
 
     def get_dimensions(self):
         return Frozen(self.ds.dimensions)
