@@ -1,5 +1,7 @@
 import os
+import warnings
 from collections import OrderedDict
+from distutils.version import LooseVersion
 import numpy as np
 
 from .. import DataArray
@@ -20,6 +22,7 @@ _ERROR_MSG = ('The kind of indexing operation you are trying to do is not '
 
 class RasterioArrayWrapper(BackendArray):
     """A wrapper around rasterio dataset objects"""
+
     def __init__(self, rasterio_ds):
         self.rasterio_ds = rasterio_ds
         self._shape = (rasterio_ds.count, rasterio_ds.height,
@@ -63,9 +66,9 @@ class RasterioArrayWrapper(BackendArray):
             elif is_scalar(k):
                 # windowed operations will always return an array
                 # we will have to squeeze it later
-                squeeze_axis.append(i+1)
+                squeeze_axis.append(i + 1)
                 start = k
-                stop = k+1
+                stop = k + 1
             else:
                 k = np.asarray(k)
                 start = k[0]
@@ -81,7 +84,39 @@ class RasterioArrayWrapper(BackendArray):
         return out
 
 
-def open_rasterio(filename, chunks=None, cache=None, lock=None):
+def _parse_envi(meta):
+    """Parse ENVI metadata into Python data structures.
+
+    See the link for information on the ENVI header file format:
+    http://www.harrisgeospatial.com/docs/enviheaderfiles.html
+
+    Parameters
+    ----------
+    meta : dict
+        Dictionary of keys and str values to parse, as returned by the rasterio
+        tags(ns='ENVI') call.
+
+    Returns
+    -------
+    parsed_meta : dict
+        Dictionary containing the original keys and the parsed values
+
+    """
+
+    def parsevec(s):
+        return np.fromstring(s.strip('{}'), dtype='float', sep=',')
+
+    def default(s):
+        return s.strip('{}')
+
+    parse = {'wavelength': parsevec,
+             'fwhm': parsevec}
+    parsed_meta = {k: parse.get(k, default)(v) for k, v in meta.items()}
+    return parsed_meta
+
+
+def open_rasterio(filename, parse_coordinates=None, chunks=None, cache=None,
+                  lock=None):
     """Open a file with rasterio (experimental).
 
     This should work with any file that rasterio can open (most often:
@@ -91,15 +126,25 @@ def open_rasterio(filename, chunks=None, cache=None, lock=None):
     <http://web.archive.org/web/20160326194152/http://remotesensing.org/geotiff/spec/geotiff2.5.html#2.5.2>`_
     for more information).
 
+    You can generate 2D coordinates from the file's attributes with::
+
+        from affine import Affine
+        da = xr.open_rasterio('path_to_file.tif')
+        transform = Affine(*da.attrs['transform'])
+        nx, ny = da.sizes['x'], da.sizes['y']
+        x, y = np.meshgrid(np.arange(nx)+0.5, np.arange(ny)+0.5) * transform
+
+
     Parameters
     ----------
     filename : str
         Path to the file to open.
-
-    Returns
-    -------
-    data : DataArray
-        The newly created DataArray.
+    parse_coordinates : bool, optional
+        Whether to parse the x and y coordinates out of the file's
+        ``transform`` attribute or not. The default is to automatically
+        parse the coordinates only if they are rectilinear (1D).
+        It can be useful to set ``parse_coordinates=False``
+        if your files are very large or if you don't need the coordinates.
     chunks : int, tuple or dict, optional
         Chunk sizes along each dimension, e.g., ``5``, ``(5, 5)`` or
         ``{'x': 5, 'y': 5}``. If chunks is provided, it used to load the new
@@ -114,6 +159,11 @@ def open_rasterio(filename, chunks=None, cache=None, lock=None):
         :py:func:`dask.array.from_array`. By default, a global lock is
         used to avoid issues with concurrent access to the same file when using
         dask's multithreaded backend.
+
+    Returns
+    -------
+    data : DataArray
+        The newly created DataArray.
     """
 
     import rasterio
@@ -129,18 +179,38 @@ def open_rasterio(filename, chunks=None, cache=None, lock=None):
         raise ValueError('Unknown dims')
     coords['band'] = np.asarray(riods.indexes)
 
-    # Get geo coords
-    nx, ny = riods.width, riods.height
-    dx, dy = riods.res[0], -riods.res[1]
-    x0 = riods.bounds.right if dx < 0 else riods.bounds.left
-    y0 = riods.bounds.top if dy < 0 else riods.bounds.bottom
-    coords['y'] = np.linspace(start=y0 + dy/2, num=ny,
-                              stop=(y0 + (ny - 1) * dy) + dy/2)
-    coords['x'] = np.linspace(start=x0 + dx/2, num=nx,
-                              stop=(x0 + (nx - 1) * dx) + dx/2)
+    # Get coordinates
+    if LooseVersion(rasterio.__version__) < '1.0':
+        transform = riods.affine
+    else:
+        transform = riods.transform
+    if transform.is_rectilinear:
+        # 1d coordinates
+        parse = True if parse_coordinates is None else parse_coordinates
+        if parse:
+            nx, ny = riods.width, riods.height
+            # xarray coordinates are pixel centered
+            x, _ = (np.arange(nx) + 0.5, np.zeros(nx) + 0.5) * transform
+            _, y = (np.zeros(ny) + 0.5, np.arange(ny) + 0.5) * transform
+            coords['y'] = y
+            coords['x'] = x
+    else:
+        # 2d coordinates
+        parse = False if (parse_coordinates is None) else parse_coordinates
+        if parse:
+            warnings.warn("The file coordinates' transformation isn't "
+                          "rectilinear: xarray won't parse the coordinates "
+                          "in this case. Set `parse_coordinates=False` to "
+                          "suppress this warning.",
+                          RuntimeWarning, stacklevel=3)
 
     # Attributes
-    attrs = {}
+    attrs = dict()
+    # Affine transformation matrix (always available)
+    # This describes coefficients mapping pixel coordinates to CRS
+    # For serialization store as tuple of 6 floats, the last row being
+    # always (0, 0, 1) per definition (see https://github.com/sgillies/affine)
+    attrs['transform'] = tuple(transform)[:6]
     if hasattr(riods, 'crs') and riods.crs:
         # CRS is a dict-like object specific to rasterio
         # If CRS is not None, we convert it back to a PROJ4 string using
@@ -157,6 +227,25 @@ def open_rasterio(filename, chunks=None, cache=None, lock=None):
         # Affine transformation matrix (tuple of floats)
         # Describes coefficients mapping pixel coordinates to CRS
         attrs['transform'] = tuple(riods.transform)
+    if hasattr(riods, 'nodatavals'):
+        # The nodata values for the raster bands
+        attrs['nodatavals'] = tuple([np.nan if nodataval is None else nodataval
+                                     for nodataval in riods.nodatavals])
+
+    # Parse extra metadata from tags, if supported
+    parsers = {'ENVI': _parse_envi}
+
+    driver = riods.driver
+    if driver in parsers:
+        meta = parsers[driver](riods.tags(ns=driver))
+
+        for k, v in meta.items():
+            # Add values as coordinates if they match the band count,
+            # as attributes otherwise
+            if isinstance(v, (list, np.ndarray)) and len(v) == riods.count:
+                coords[k] = ('band', np.asarray(v))
+            else:
+                attrs[k] = v
 
     data = indexing.LazilyIndexedArray(RasterioArrayWrapper(riods))
 
@@ -171,7 +260,11 @@ def open_rasterio(filename, chunks=None, cache=None, lock=None):
     if chunks is not None:
         from dask.base import tokenize
         # augment the token with the file modification time
-        mtime = os.path.getmtime(filename)
+        try:
+            mtime = os.path.getmtime(filename)
+        except OSError:
+            # the filename is probably an s3 bucket rather than a regular file
+            mtime = None
         token = tokenize(filename, mtime, chunks)
         name_prefix = 'open_rasterio-%s' % token
         if lock is None:
