@@ -4,7 +4,7 @@ from __future__ import print_function
 import numpy as np
 
 try:
-    from numpy import nancumsum, nancumprod
+    from numpy import nancumsum, nancumprod, flip
 except ImportError:  # pragma: no cover
     # Code copied from newer versions of NumPy (v1.12).
     # Used under the terms of NumPy's license, see licenses/NUMPY_LICENSE.
@@ -33,8 +33,8 @@ except ImportError:  # pragma: no cover
             If `a` is of inexact type, return a copy of `a` with the NaNs
             replaced by the fill value, otherwise return `a`.
         mask: {bool, None}
-            If `a` is of inexact type, return a boolean mask marking locations of
-            NaNs, otherwise return None.
+            If `a` is of inexact type, return a boolean mask marking locations
+            of NaNs, otherwise return None.
 
         """
         is_new = not isinstance(a, np.ndarray)
@@ -174,3 +174,74 @@ except ImportError:  # pragma: no cover
         """
         a, mask = _replace_nan(a, 1)
         return np.cumprod(a, axis=axis, dtype=dtype, out=out)
+
+    def flip(m, axis):
+        """
+        Reverse the order of elements in an array along the given axis.
+
+        The shape of the array is preserved, but the elements are reordered.
+
+        .. versionadded:: 1.12.0
+
+        Parameters
+        ----------
+        m : array_like
+            Input array.
+        axis : integer
+            Axis in array, which entries are reversed.
+
+
+        Returns
+        -------
+        out : array_like
+            A view of `m` with the entries of axis reversed.  Since a view is
+            returned, this operation is done in constant time.
+
+        See Also
+        --------
+        flipud : Flip an array vertically (axis=0).
+        fliplr : Flip an array horizontally (axis=1).
+
+        Notes
+        -----
+        flip(m, 0) is equivalent to flipud(m).
+        flip(m, 1) is equivalent to fliplr(m).
+        flip(m, n) corresponds to ``m[...,::-1,...]`` with ``::-1`` at index n.
+
+        Examples
+        --------
+        >>> A = np.arange(8).reshape((2,2,2))
+        >>> A
+        array([[[0, 1],
+                [2, 3]],
+
+               [[4, 5],
+                [6, 7]]])
+
+        >>> flip(A, 0)
+        array([[[4, 5],
+                [6, 7]],
+
+               [[0, 1],
+                [2, 3]]])
+
+        >>> flip(A, 1)
+        array([[[2, 3],
+                [0, 1]],
+
+               [[6, 7],
+                [4, 5]]])
+
+        >>> A = np.random.randn(3,4,5)
+        >>> np.all(flip(A,2) == A[:,:,::-1,...])
+        True
+        """
+        if not hasattr(m, 'ndim'):
+            m = np.asarray(m)
+        indexer = [slice(None)] * m.ndim
+        try:
+            indexer[axis] = slice(None, None, -1)
+        except IndexError:
+            raise ValueError("axis=%i is invalid for the %i-dimensional "
+                             "input array" % (axis, m.ndim))
+        return m[tuple(indexer)]
