@@ -12,7 +12,6 @@ from __future__ import print_function
 import operator
 
 import numpy as np
-import pandas as pd
 
 from . import dtypes
 from . import duck_array_ops
@@ -262,7 +261,7 @@ def inject_reduce_methods(cls):
 
 def inject_cum_methods(cls):
     methods = ([(name, getattr(duck_array_ops, name), True)
-               for name in NAN_CUM_METHODS])
+                for name in NAN_CUM_METHODS])
     for name, f, include_skipna in methods:
         numeric_only = getattr(f, 'numeric_only', False)
         func = cls._reduce_method(f, include_skipna, numeric_only)
@@ -320,7 +319,8 @@ def inject_all_ops_and_reduce_methods(cls, priority=50, array_only=True):
         setattr(cls, name, cls._unary_op(_method_wrapper(name)))
 
     for name in PANDAS_UNARY_FUNCTIONS:
-        f = _func_slash_method_wrapper(getattr(pd, name), name=name)
+        f = _func_slash_method_wrapper(
+            getattr(duck_array_ops, name), name=name)
         setattr(cls, name, cls._unary_op(f))
 
     f = _func_slash_method_wrapper(duck_array_ops.around, name='round')
