@@ -14,13 +14,15 @@ import xarray as xr
 from xarray import Variable, DataArray, Dataset
 import xarray.ufuncs as xu
 from xarray.core.pycompat import suppress, OrderedDict
-from . import TestCase, assert_frame_equal, raises_regex
+from . import (
+    TestCase, assert_frame_equal, raises_regex, assert_equal, assert_identical,
+    assert_array_equal, assert_allclose)
 
 from xarray.tests import mock
 
 dask = pytest.importorskip('dask')
-import dask.array as da
-import dask.dataframe as dd
+import dask.array as da  # noqa: E402  # allow importorskip call above this
+import dask.dataframe as dd  # noqa: E402
 
 
 class DaskTestCase(TestCase):
@@ -30,28 +32,28 @@ class DaskTestCase(TestCase):
         if isinstance(actual, Dataset):
             for k, v in actual.variables.items():
                 if k in actual.dims:
-                    self.assertIsInstance(v.data, np.ndarray)
+                    assert isinstance(v.data, np.ndarray)
                 else:
-                    self.assertIsInstance(v.data, da.Array)
+                    assert isinstance(v.data, da.Array)
         elif isinstance(actual, DataArray):
-            self.assertIsInstance(actual.data, da.Array)
+            assert isinstance(actual.data, da.Array)
             for k, v in actual.coords.items():
                 if k in actual.dims:
-                    self.assertIsInstance(v.data, np.ndarray)
+                    assert isinstance(v.data, np.ndarray)
                 else:
-                    self.assertIsInstance(v.data, da.Array)
+                    assert isinstance(v.data, da.Array)
         elif isinstance(actual, Variable):
-            self.assertIsInstance(actual.data, da.Array)
+            assert isinstance(actual.data, da.Array)
         else:
             assert False
 
 
 class TestVariable(DaskTestCase):
     def assertLazyAndIdentical(self, expected, actual):
-        self.assertLazyAnd(expected, actual, self.assertVariableIdentical)
+        self.assertLazyAnd(expected, actual, assert_identical)
 
     def assertLazyAndAllClose(self, expected, actual):
-        self.assertLazyAnd(expected, actual, self.assertVariableAllClose)
+        self.assertLazyAnd(expected, actual, assert_allclose)
 
     def setUp(self):
         self.values = np.random.RandomState(0).randn(4, 6)
@@ -62,9 +64,9 @@ class TestVariable(DaskTestCase):
 
     def test_basics(self):
         v = self.lazy_var
-        self.assertIs(self.data, v.data)
-        self.assertEqual(self.data.chunks, v.chunks)
-        self.assertArrayEqual(self.values, v)
+        assert self.data is v.data
+        assert self.data.chunks == v.chunks
+        assert_array_equal(self.values, v)
 
     def test_copy(self):
         self.assertLazyAndIdentical(self.eager_var, self.lazy_var.copy())
@@ -78,7 +80,7 @@ class TestVariable(DaskTestCase):
                                  ({'x': 3}, ((3, 1), (2, 2, 2))),
                                  ({'x': (3, 1)}, ((3, 1), (2, 2, 2)))]:
             rechunked = self.lazy_var.chunk(chunks)
-            self.assertEqual(rechunked.chunks, expected)
+            assert rechunked.chunks == expected
             self.assertLazyAndIdentical(self.eager_var, rechunked)
 
     def test_indexing(self):
@@ -97,10 +99,10 @@ class TestVariable(DaskTestCase):
 
     def test_equals(self):
         v = self.lazy_var
-        self.assertTrue(v.equals(v))
-        self.assertIsInstance(v.data, da.Array)
-        self.assertTrue(v.identical(v))
-        self.assertIsInstance(v.data, da.Array)
+        assert v.equals(v)
+        assert isinstance(v.data, da.Array)
+        assert v.identical(v)
+        assert isinstance(v.data, da.Array)
 
     def test_transpose(self):
         u = self.eager_var
@@ -112,13 +114,13 @@ class TestVariable(DaskTestCase):
         v = self.lazy_var
         self.assertLazyAndIdentical(u.shift(x=2), v.shift(x=2))
         self.assertLazyAndIdentical(u.shift(x=-2), v.shift(x=-2))
-        self.assertEqual(v.data.chunks, v.shift(x=1).data.chunks)
+        assert v.data.chunks == v.shift(x=1).data.chunks
 
     def test_roll(self):
         u = self.eager_var
         v = self.lazy_var
         self.assertLazyAndIdentical(u.roll(x=2), v.roll(x=2))
-        self.assertEqual(v.data.chunks, v.roll(x=1).data.chunks)
+        assert v.data.chunks == v.roll(x=1).data.chunks
 
     def test_unary_op(self):
         u = self.eager_var
@@ -138,20 +140,20 @@ class TestVariable(DaskTestCase):
         expected = dedent("""\
         <xarray.Variable (x: 4, y: 6)>
         dask.array<shape=(4, 6), dtype=float64, chunksize=(2, 2)>""")
-        self.assertEqual(expected, repr(self.lazy_var))
+        assert expected == repr(self.lazy_var)
 
     def test_pickle(self):
         # Test that pickling/unpickling does not convert the dask
         # backend to numpy
         a1 = Variable(['x'], build_dask_array('x'))
         a1.compute()
-        self.assertFalse(a1._in_memory)
+        assert not a1._in_memory
         assert kernel_call_count == 1
         a2 = pickle.loads(pickle.dumps(a1))
         assert kernel_call_count == 1
-        self.assertVariableIdentical(a1, a2)
-        self.assertFalse(a1._in_memory)
-        self.assertFalse(a2._in_memory)
+        assert_identical(a1, a2)
+        assert not a1._in_memory
+        assert not a2._in_memory
 
     def test_reduce(self):
         u = self.eager_var
@@ -171,7 +173,8 @@ class TestVariable(DaskTestCase):
         eager_var = Variable('x', values)
         lazy_var = Variable('x', data)
         self.assertLazyAndIdentical(eager_var, lazy_var.fillna(lazy_var))
-        self.assertLazyAndIdentical(Variable('x', range(4)), lazy_var.fillna(2))
+        self.assertLazyAndIdentical(Variable('x', range(4)),
+                                    lazy_var.fillna(2))
         self.assertLazyAndIdentical(eager_var.count(), lazy_var.count())
 
     def test_concat(self):
@@ -182,18 +185,19 @@ class TestVariable(DaskTestCase):
         self.assertLazyAndIdentical(u[:2], Variable.concat([u[0], v[1]], 'x'))
         self.assertLazyAndIdentical(u[:2], Variable.concat([v[0], u[1]], 'x'))
         self.assertLazyAndIdentical(
-            u[:3], Variable.concat([v[[0, 2]], v[[1]]], 'x', positions=[[0, 2], [1]]))
+            u[:3],
+            Variable.concat([v[[0, 2]], v[[1]]], 'x', positions=[[0, 2], [1]]))
 
     def test_missing_methods(self):
         v = self.lazy_var
         try:
             v.argsort()
         except NotImplementedError as err:
-            self.assertIn('dask', str(err))
+            assert 'dask' in str(err)
         try:
             v[0].item()
         except NotImplementedError as err:
-            self.assertIn('dask', str(err))
+            assert 'dask' in str(err)
 
     def test_univariate_ufunc(self):
         u = self.eager_var
@@ -237,13 +241,13 @@ class TestVariable(DaskTestCase):
 
 class TestDataArrayAndDataset(DaskTestCase):
     def assertLazyAndIdentical(self, expected, actual):
-        self.assertLazyAnd(expected, actual, self.assertDataArrayIdentical)
+        self.assertLazyAnd(expected, actual, assert_identical)
 
     def assertLazyAndAllClose(self, expected, actual):
-        self.assertLazyAnd(expected, actual, self.assertDataArrayAllClose)
+        self.assertLazyAnd(expected, actual, assert_allclose)
 
     def assertLazyAndEqual(self, expected, actual):
-        self.assertLazyAnd(expected, actual, self.assertDataArrayEqual)
+        self.assertLazyAnd(expected, actual, assert_equal)
 
     def setUp(self):
         self.values = np.random.randn(4, 6)
@@ -255,16 +259,16 @@ class TestDataArrayAndDataset(DaskTestCase):
 
     def test_rechunk(self):
         chunked = self.eager_array.chunk({'x': 2}).chunk({'y': 2})
-        self.assertEqual(chunked.chunks, ((2,) * 2, (2,) * 3))
+        assert chunked.chunks == ((2,) * 2, (2,) * 3)
         self.assertLazyAndIdentical(self.lazy_array, chunked)
 
     def test_new_chunk(self):
         chunked = self.eager_array.chunk()
-        self.assertTrue(chunked.data.name.startswith('xarray-<this-array>'))
+        assert chunked.data.name.startswith('xarray-<this-array>')
 
     def test_lazy_dataset(self):
         lazy_ds = Dataset({'foo': (('x', 'y'), self.data)})
-        self.assertIsInstance(lazy_ds.foo.variable.data, da.Array)
+        assert isinstance(lazy_ds.foo.variable.data, da.Array)
 
     def test_lazy_array(self):
         u = self.eager_array
@@ -331,13 +335,15 @@ class TestDataArrayAndDataset(DaskTestCase):
         assert isinstance(out['d'].data, np.ndarray)
         assert isinstance(out['c'].data, np.ndarray)
 
-        out = xr.concat([ds1, ds2, ds3], dim='n', data_vars='all', coords='all')
+        out = xr.concat(
+            [ds1, ds2, ds3], dim='n', data_vars='all', coords='all')
         # no extra kernel calls
         assert kernel_call_count == 6
         assert isinstance(out['d'].data, dask.array.Array)
         assert isinstance(out['c'].data, dask.array.Array)
 
-        out = xr.concat([ds1, ds2, ds3], dim='n', data_vars=['d'], coords=['c'])
+        out = xr.concat(
+            [ds1, ds2, ds3], dim='n', data_vars=['d'], coords=['c'])
         # no extra kernel calls
         assert kernel_call_count == 6
         assert isinstance(out['d'].data, dask.array.Array)
@@ -358,7 +364,8 @@ class TestDataArrayAndDataset(DaskTestCase):
 
         # When the test for different turns true halfway through,
         # stop computing variables as it would not have any benefit
-        ds4 = Dataset(data_vars={'d': ('x', [2.0])}, coords={'c': ('x', [2.0])})
+        ds4 = Dataset(data_vars={'d': ('x', [2.0])},
+                      coords={'c': ('x', [2.0])})
         out = xr.concat([ds1, ds2, ds4, ds3], dim='n', data_vars='different',
                         coords='different')
         # the variables of ds1 and ds2 were computed, but those of ds3 didn't
@@ -456,7 +463,7 @@ class TestDataArrayAndDataset(DaskTestCase):
 
         with dask.set_options(get=counting_get):
             ds.load()
-        self.assertEqual(count[0], 1)
+        assert count[0] == 1
 
     def test_stack(self):
         data = da.random.normal(size=(2, 3, 4), chunks=(1, 3, 4))
@@ -485,7 +492,7 @@ class TestDataArrayAndDataset(DaskTestCase):
         Coordinates:
             y        (x) int64 dask.array<shape=(1,), chunksize=(1,)>
         Dimensions without coordinates: x""")
-        self.assertEqual(expected, repr(a))
+        assert expected == repr(a)
         assert kernel_call_count == 0
 
     def test_dataset_repr(self):
@@ -503,7 +510,7 @@ class TestDataArrayAndDataset(DaskTestCase):
         Dimensions without coordinates: x
         Data variables:
             a        (x) int64 dask.array<shape=(1,), chunksize=(1,)>""")
-        self.assertEqual(expected, repr(ds))
+        assert expected == repr(ds)
         assert kernel_call_count == 0
 
     def test_dataarray_pickle(self):
@@ -513,16 +520,16 @@ class TestDataArrayAndDataset(DaskTestCase):
         nonindex_coord = build_dask_array('coord')
         a1 = DataArray(data, dims=['x'], coords={'y': ('x', nonindex_coord)})
         a1.compute()
-        self.assertFalse(a1._in_memory)
-        self.assertFalse(a1.coords['y']._in_memory)
+        assert not a1._in_memory
+        assert not a1.coords['y']._in_memory
         assert kernel_call_count == 2
         a2 = pickle.loads(pickle.dumps(a1))
         assert kernel_call_count == 2
-        self.assertDataArrayIdentical(a1, a2)
-        self.assertFalse(a1._in_memory)
-        self.assertFalse(a2._in_memory)
-        self.assertFalse(a1.coords['y']._in_memory)
-        self.assertFalse(a2.coords['y']._in_memory)
+        assert_identical(a1, a2)
+        assert not a1._in_memory
+        assert not a2._in_memory
+        assert not a1.coords['y']._in_memory
+        assert not a2.coords['y']._in_memory
 
     def test_dataset_pickle(self):
         # Test that pickling/unpickling converts the dask backend
@@ -532,16 +539,16 @@ class TestDataArrayAndDataset(DaskTestCase):
         ds1 = Dataset(data_vars={'a': ('x', data)},
                       coords={'y': ('x', nonindex_coord)})
         ds1.compute()
-        self.assertFalse(ds1['a']._in_memory)
-        self.assertFalse(ds1['y']._in_memory)
+        assert not ds1['a']._in_memory
+        assert not ds1['y']._in_memory
         assert kernel_call_count == 2
         ds2 = pickle.loads(pickle.dumps(ds1))
         assert kernel_call_count == 2
-        self.assertDatasetIdentical(ds1, ds2)
-        self.assertFalse(ds1['a']._in_memory)
-        self.assertFalse(ds2['a']._in_memory)
-        self.assertFalse(ds1['y']._in_memory)
-        self.assertFalse(ds2['y']._in_memory)
+        assert_identical(ds1, ds2)
+        assert not ds1['a']._in_memory
+        assert not ds2['a']._in_memory
+        assert not ds1['y']._in_memory
+        assert not ds2['y']._in_memory
 
     def test_dataarray_getattr(self):
         # ipython/jupyter does a long list of getattr() calls to when trying to
@@ -570,9 +577,9 @@ class TestDataArrayAndDataset(DaskTestCase):
         # Test that invoking the values property does not convert the dask
         # backend to numpy
         a = DataArray([1, 2]).chunk()
-        self.assertFalse(a._in_memory)
+        assert not a._in_memory
         assert a.values.tolist() == [1, 2]
-        self.assertFalse(a._in_memory)
+        assert not a._in_memory
 
     def test_from_dask_variable(self):
         # Test array creation from Variable with dask backend.
@@ -602,7 +609,7 @@ class TestToDaskDataFrame(TestCase):
         expected = dd.from_pandas(expected_pd, chunksize=4)
         actual = ds.to_dask_dataframe(set_index=True)
         # test if we have dask dataframes
-        self.assertIsInstance(actual, dd.DataFrame)
+        assert isinstance(actual, dd.DataFrame)
 
         # use the .equals from pandas to check dataframes are equivalent
         assert_frame_equal(expected.compute(), actual.compute())
@@ -613,7 +620,7 @@ class TestToDaskDataFrame(TestCase):
 
         actual = ds.to_dask_dataframe(set_index=False)
 
-        self.assertIsInstance(actual, dd.DataFrame)
+        assert isinstance(actual, dd.DataFrame)
         assert_frame_equal(expected.compute(), actual.compute())
 
     def test_to_dask_dataframe_2D(self):
@@ -634,7 +641,7 @@ class TestToDaskDataFrame(TestCase):
         expected = expected.reset_index(drop=False)
         actual = ds.to_dask_dataframe(set_index=False)
 
-        self.assertIsInstance(actual, dd.DataFrame)
+        assert isinstance(actual, dd.DataFrame)
         assert_frame_equal(expected, actual.compute())
 
     @pytest.mark.xfail(raises=NotImplementedError)
@@ -647,13 +654,13 @@ class TestToDaskDataFrame(TestCase):
 
         expected = ds.compute().to_dataframe()
         actual = ds.to_dask_dataframe(set_index=True)
-        self.assertIsInstance(actual, dd.DataFrame)
+        assert isinstance(actual, dd.DataFrame)
         assert_frame_equal(expected, actual.compute())
 
     def test_to_dask_dataframe_coordinates(self):
         # Test if coordinate is also a dask array
         x = da.from_array(np.random.randn(10), chunks=4)
-        t = da.from_array(np.arange(10)*2, chunks=4)
+        t = da.from_array(np.arange(10) * 2, chunks=4)
 
         ds = Dataset(OrderedDict([('a', ('t', x)),
                                   ('t', ('t', t))]))
@@ -662,7 +669,7 @@ class TestToDaskDataFrame(TestCase):
                                    index=pd.Index(t, name='t'))
         expected = dd.from_pandas(expected_pd, chunksize=4)
         actual = ds.to_dask_dataframe(set_index=True)
-        self.assertIsInstance(actual, dd.DataFrame)
+        assert isinstance(actual, dd.DataFrame)
         assert_frame_equal(expected.compute(), actual.compute())
 
     def test_to_dask_dataframe_not_daskarray(self):
@@ -679,7 +686,7 @@ class TestToDaskDataFrame(TestCase):
                                 index=pd.Index(t, name='t'))
 
         actual = ds.to_dask_dataframe(set_index=True)
-        self.assertIsInstance(actual, dd.DataFrame)
+        assert isinstance(actual, dd.DataFrame)
         assert_frame_equal(expected, actual.compute())
 
     def test_to_dask_dataframe_no_coordinate(self):
@@ -688,12 +695,12 @@ class TestToDaskDataFrame(TestCase):
 
         expected = ds.compute().to_dataframe().reset_index()
         actual = ds.to_dask_dataframe()
-        self.assertIsInstance(actual, dd.DataFrame)
+        assert isinstance(actual, dd.DataFrame)
         assert_frame_equal(expected, actual.compute())
 
         expected = ds.compute().to_dataframe()
         actual = ds.to_dask_dataframe(set_index=True)
-        self.assertIsInstance(actual, dd.DataFrame)
+        assert isinstance(actual, dd.DataFrame)
         assert_frame_equal(expected, actual.compute())
 
     def test_to_dask_dataframe_dim_order(self):
@@ -702,12 +709,12 @@ class TestToDaskDataFrame(TestCase):
 
         expected = ds['w'].to_series().reset_index()
         actual = ds.to_dask_dataframe(dim_order=['x', 'y'])
-        self.assertIsInstance(actual, dd.DataFrame)
+        assert isinstance(actual, dd.DataFrame)
         assert_frame_equal(expected, actual.compute())
 
         expected = ds['w'].T.to_series().reset_index()
         actual = ds.to_dask_dataframe(dim_order=['y', 'x'])
-        self.assertIsInstance(actual, dd.DataFrame)
+        assert isinstance(actual, dd.DataFrame)
         assert_frame_equal(expected, actual.compute())
 
         with raises_regex(ValueError, 'does not match the set of dimensions'):
@@ -829,4 +836,18 @@ def test_dataarray_with_dask_coords():
     (array2,) = dask.compute(array)
     assert not dask.is_dask_collection(array2)
 
-    assert all(isinstance(v._variable.data, np.ndarray) for v in array2.coords.values())
+    assert all(isinstance(v._variable.data, np.ndarray)
+               for v in array2.coords.values())
+
+
+def test_basic_compute():
+    ds = Dataset({'foo': ('x', range(5)),
+                  'bar': ('x', range(5))}).chunk({'x': 2})
+    for get in [dask.threaded.get,
+                dask.multiprocessing.get,
+                dask.local.get_sync,
+                None]:
+        with dask.set_options(get=get):
+            ds.compute()
+            ds.foo.compute()
+            ds.foo.variable.compute()
